/******************************  Пречисления  **********************************/
/// <summary>
/// Перечисление - Тип протокола передачи данных
/// </summary>
enum ConnectionTypeEnum
{
	/// <summary>
	/// Не выбран
	/// </summary>
	CT_None = 0,   
	/// <summary>
	/// Serial порт в обе стороны
	/// </summary>
	CT_Serial = 1,
	/// <summary>
	/// UDP
	/// </summary>	
	CT_UDP = 2
};

/// <summary>
/// Перечисление - Тип операции 
/// </summary>
enum OperationTypeEnum     
{
	/// <summary>
	/// Не выбрана
	/// </summary>
	OT_None = 0,
	/// <summary>
	/// Проверка какая ардуина подключена, запрос на запуск ответа
	/// </summary>
	OT_CheckWhatKindArduinConnectedStart = 1,
	/// <summary>
	/// Проверка какая ардуина подключена, ответ от ардуины
	/// </summary>
	OT_CheckWhatKindArduinConnected = 2,
	/// <summary>
	/// Проверка какая ардуина подключена, запрос на остановку ответа
	/// </summary>
	OT_CheckWhatKindArduinConnectedStop = 3,
	/// <summary>
	/// Старт протокола передачи данных
	/// </summary>
	OT_ProtocolDataTransferStart = 4,
	/// <summary>
	/// Остановка протокола передачи данных
	/// </summary>
	OT_ProtocolDataTransferStop = 5,
	/// <summary>
	/// Проверка протокола передачи данных
	/// </summary>
	OT_CheckProtocolDataTransfer = 6,
	/// <summary>
	/// Старт инициализации Arduino
	/// </summary>
	OT_InitArduinoStart = 8,
	/// <summary>
	/// Инициализация Arduino
	/// </summary>
	OT_InitArduinoWork = 9,
	/// <summary>
	/// Остановка инициализации Arduino
	/// </summary>
	OT_InitArduinoStop = 10,
	/// <summary>
	/// TPA готов к работе
	/// </summary>
	OT_TPAReadyToWork = 11,
	/// <summary>
	/// Старт ТПА
	/// </summary>
	OT_StartTPA = 12,
	/// <summary>
	/// Стоп ТПА
	/// </summary>
	OT_StopTPA = 13,
	/// <summary>
	/// Отладочное сообщение с Arduino
	/// </summary>
	OT_DebugMessageFromArduino = 14,
	/// <summary>
	/// Отладочное сообщение с PC
	/// </summary>
	OT_DebugMessageFromPC = 15,
	/// <summary>
	/// Ошибка на стороне Arduino
	/// </summary>
	OT_ErrorMessageFromArduino = 16,
	/// <summary>
	/// Ошибка на стороне PC
	/// </summary>
	OT_ErrorMessageFromPC = 17
};

/// <summary>
/// Перечисление - Тип параметра
/// </summary>
enum ParametrEnum
{
	/// <summary>
	/// Не установлен
	/// </summary>
	P_None = 0,
	/// <summary>
	/// V1
	/// </summary>
	P_V1 = 1,
	/// <summary>
	/// V2
	/// </summary>
	P_V2 = 2,
	/// <summary>
	/// V3
	/// </summary>
	P_V3 = 3,
	/// <summary>
	/// V4
	/// </summary>
	P_V4 = 4,
	/// <summary>
	/// V5
	/// </summary>
	P_V5 = 5,
	/// <summary>
	/// V6
	/// </summary>
	P_V6 = 6,
	/// <summary>
	/// V7
	/// </summary>
	P_V7 = 7,
	/// <summary>
	/// V2
	/// </summary>
	P_V8 = 8,
	/// <summary>
	/// V9
	/// </summary>
	P_V9 = 9,
	/// <summary>
	/// V10
	/// </summary>
	P_V10 = 10,
	/// <summary>
	/// V11
	/// </summary>
	P_V11 = 11,
	/// <summary>
	/// P1
	/// </summary>
	P_P1 = 12,
	/// <summary>
	/// P2
	/// </summary>
	P_P2 = 13,
	/// <summary>
	/// P3
	/// </summary>
	P_P3 = 14,
	/// <summary>
	/// P4
	/// </summary>
	P_P4 = 15,
	/// <summary>
	/// P5
	/// </summary>
	P_P5 = 16,
	/// <summary>
	/// P6
	/// </summary>
	P_P6 = 17,
	/// <summary>
	/// P7
	/// </summary>
	P_P7 = 18,
	/// <summary>
	/// P8
	/// </summary>
	P_P8 = 19,
	/// <summary>
	/// P9
	/// </summary>
	P_P9 = 20,
	/// <summary>
	/// P10
	/// </summary>
	P_P10 = 21,
	/// <summary>
	/// T1
	/// </summary>
	P_T1 = 22,
	/// <summary>
	/// T2
	/// </summary>
	P_T2 = 23,
	/// <summary>
	/// T3
	/// </summary>
	P_T3 = 24,
	/// <summary>
	/// T4
	/// </summary>
	P_T4 = 25,
	/// <summary>
	/// T5
	/// </summary>
	P_T5 = 26,
	/// <summary>
	/// T6
	/// </summary>
	P_T6 = 27,
	/// <summary>
	/// T7
	/// </summary>
	P_T7 = 28,
	/// <summary>
	/// T8
	/// </summary>
	P_T8 = 29,
	/// <summary>
	/// TL
	/// </summary>
	P_TL = 30
};

/// <summary>
/// Перечисление - Тип  данных параметра
/// </summary>
enum DataTypeEnum 
{
	/// <summary>
	/// Не указан
	/// </summary>
	DT_None = 0,
	/// <summary>
	/// Int
	/// </summary>
	DT_Int = 1
};

/// <summary>
/// Тип работы ТПА
/// </summary>
enum TPAWorkEnum
{
	/// <summary>
	/// Не выбран
	/// </summary>
	TPAW_None = 0,
	/// <summary>
	/// Автоматический режим работы
	/// </summary>
	TPAW_Automatic = 1,
	/// <summary>
	/// Полуавтоматический режим работы
	/// </summary>
	TPAW_SemiAutomatic = 2,
	/// <summary>
	/// Ручной режим работы
	/// </summary>
	TPAW_Manual = 3
};

/// <summary>
/// Тип шага в автоматическом цикле работы ТПА
/// </summary>
enum TPAAutomaticStepCycleEnum
{
	/// <summary>
	/// Не выбран
	/// </summary>
	TPAASC_None = 0,
	/// <summary>
	/// Смыкание формы
	/// </summary>
	TPAASC_CloseForm = 1,
	/// <summary>
	/// Выс. давл. дожатия
	/// </summary>
	TPAASC_CloseFormHighPressureSqueezing = 2,
	/// <summary>
	/// Защита формы
	/// </summary>
	TPAASC_DefenceFrom = 3,
	/// <summary>
	/// Подвод сопла
	/// </summary>
	TPAASC_SupplyNozzle = 4,
	/// <summary>
	/// Подвод шнека
	/// </summary>
	TPAASC_SupplyAuger = 5,
	/// <summary>
	/// Впрыск 1
	/// </summary>
	TPAASC_Injection1 = 6,
	/// <summary>
	/// Впрыск 2
	/// </summary>
	TPAASC_Injection2 = 7,
	/// <summary>
	/// Впрыск 3
	/// </summary>
	TPAASC_Injection3 = 8,
	/// <summary>
	/// Выдержка 1
	/// </summary>
	TPAASC_RetentionInterval1 = 9,
	/// <summary>
	/// Выдержка 2
	/// </summary>
	TPAASC_RetentionInterval2 = 10,
	/// <summary>
	/// Выдержка 3
	/// </summary>
	TPAASC_RetentionInterval3 = 11,
	/// <summary>
	/// Загрузка
	/// </summary>
	TPAASC_LoadFromBunker = 12,
	/// <summary>
	/// Отвод шнека
	/// </summary>
	TPAASC_DiversionAuger = 13,
	/// <summary>
	/// Отвод сопла
	/// </summary>
	TPAASC_DiversionNozzle = 14,
	/// <summary>
	/// Выс. давл. раскр. формы
	/// </summary>
	TPAASC_OpenFormHighPressureDisclosure = 15,
	/// <summary>
	/// Открытие формы
	/// </summary>
	TPAASC_OpenForm = 16,
	/// <summary>
	/// Открытие формы стоп
	/// </summary>
	TPAASC_OpenFormStop = 17,
	/// <summary>
	/// Толкатель вперёд
	/// </summary>
	TPAASC_PlungerForward = 18
};

/// <summary>
/// Тип шага в полу-автоматическом цикле работы ТПА
/// </summary>
enum TPASemiAutomaticStepCycleEnum
{
	/// <summary>
	/// Не выбран
	/// </summary>
	TPASASC_None = 0
};

/// <summary>
/// Тип шага в ручном цикле работы ТПА
/// </summary>
enum TPAManualStepCycleEnum
{
	/// <summary>
	/// Не выбран
	/// </summary>
	TPAMSC_None = 0,
	/// <summary>
	/// Смыкание формы
	/// </summary>
	TPAMSC_CloseForm = 1,
	/// <summary>
	/// Подвод сопла
	/// </summary>
	TPAMSC_SupplyNozzle = 2,
	/// <summary>
	/// Подвод шнека
	/// </summary>
	TPAMSC_SupplyAuger = 3,
	/// <summary>
	/// Впрыск
	/// </summary>
	TPAMSC_Injection = 4,
	/// <summary>
	/// Загрузка с бункера
	/// </summary>
	TPAMSC_LoadFromBunker = 5,
	/// <summary>
	/// Отвод шнека
	/// </summary>
	TPAMSC_DiversionAuger = 6,
	/// <summary>
	/// Отвод сопла
	/// </summary>
	TPAMSC_DiversionNozzle = 7,
	/// <summary>
	/// Открытие формы
	/// </summary>
	TPAMSC_OpenForm = 8,
	/// <summary>
	/// Толкатель вперёд
	/// </summary>
	TPAMSC_PlungerForward = 9,
	/// <summary>
	/// Толкатель назад
	/// </summary>
	TPAMSC_PusherAgo = 10
};
/******************************  /Пречисления  **********************************/
