/*
Fast I2C
Может работать и на 400 кГц, но вам придется настроить библиотеку в файлах
hardware/libraries/Wire/Wire.o 
hardware/libraries/Wire/utility/twi.o :
#ifndef TWI_FREQ 
#define TWI_FREQ 100000L 
#endif
Заменить на 
#ifndef TWI_FREQ 
#define TWI_FREQ 400000L 
#endif
*/
/******************************  Библиотеки  **********************************/
#include <Keypad.h>         // Библиотека клавиатуры
#include <SPI.h>            // Библиотека шины SPI     
#include <Ethernet.h>       // Библиотека для работы с Ethernet-шилдом
#include <EthernetUdp.h>    // Библиотека для работы с UDP-пакетами   
#include "Enum.h"           // Библиотека переменных Enum
#include "PinVar.h"            // Библиотека переменных

#include <avr/io.h>
#include <avr/delay.h>
/******************************  /Библиотеки  **********************************/

/******************************  Ускоряем работу ADC  **********************************/
#define FASTADC 1
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif
/******************************  /Ускоряем работу ADC  **********************************/

/******************************  Настройка клавиатуры  **********************************/
const byte ROWS = 4; // 4 строки
const byte COLS = 4; // 4 колонки

// Указываем символы клавиатуры
char hexaKeys[ROWS][COLS] = {
  {'1','2','3','A'}, // 1 строка
  {'4','5','6','B'}, // 2 строка
  {'7','8','9','C'}, // 3 строка
  {'*','0','#','D'}  // 4 строка
};
byte rowPins[ROWS] = {23,25,27,29}; // Указываем pin-ы для строк
byte colPins[COLS] = {31,33,35,37};  // Указываем pin-ы для колонок

// Инициализация клавиатуры
Keypad customKeypad = Keypad(makeKeymap(hexaKeys), rowPins, colPins, ROWS, COLS); 
/******************************  /Настройка клавиатуры  **********************************/

/******************************  Переменные перечисления  **********************************/
/// <summary>
/// Переменная определяющая тип связи с компьютером
/// </summary>
ConnectionTypeEnum connectionType = CT_Serial; 

/// <summary>
/// Переменная хранящая тип операции
/// </summary>
OperationTypeEnum operationType = OT_None; 

/// <summary>
/// Переменная определяющая тип работы ТПА
/// </summary>
TPAWorkEnum tPAWorkEnum = TPAW_Manual;

/// <summary>
/// Переменная определяющая последний шаг ручного режима
/// </summary>
TPAManualStepCycleEnum tPAManualStepCycleEnum = TPAMSC_None; 
/******************************  /Переменные перечисления  **********************************/

/******************************  Переменные работы ТПА  **********************************/
// TODO: Добавить описание
int v1 = 0;  // v1 
int v2 = 0;  // v2
int v3 = 0;  // v3
int v4 = 0;  // v4
int v30percent = 0;  // v = 30%
int v5 = 0;  // v5
int v6 = 0;  // v6
int v7 = 0;  // v7
int v8 = 0;  // v8
int v9 = 0;  // v9
int v10 = 0; // v10
int v11 = 0; // v11

int p1 = 0;  // p1
int p2 = 0;  // p2
int p3 = 0;  // p3
int p4 = 0;  // p4
int p5 = 0;  // p5
int p6 = 0;  // p6
int p7 = 0;  // p7
int p8 = 0;  // p8
int p9 = 0;  // p9
int p10 = 0; // p10

int t1 = 0; // t1
int t2 = 0; // t2
int t3 = 0; // t3
int t4 = 0; // t4
int t5 = 0; // t5
int t6 = 0; // t6
int t7 = 0; // t7
int t8 = 0; // t8
int tl = 0; // tl

long previousT1 = 0; // t1
long previousT2 = 0; // t2
long previousT3 = 0; // t3
long previousT4 = 0; // t4
long previousT5 = 0; // t5
long previousT6 = 0; // t6
long previousT7 = 0; // t7
long previousT8 = 0; // t8
long previousTl = 0; // tl

int b22 = 10; // b22
int b23 = 40; // b23
int b24r = 0; // b24r
int b24l = 0; // b24l

int b43 = 30; // b43
int b44 = 0; // b44
int b45 = 0; // b45
int b46 = 20; // b46
int b47 = 0; // b47

int b26 = 0; // b26
int b27 = 0; // b27
int b28 = 0; // b28
int b29r = 0; // b29r
int b29l = 0; // b29l

/// <summary>
/// Переменная хранящее состояние, без подвода сопла
/// </summary>
boolean withSupplyNozzle = false;
/******************************  /Переменные работы ТПА  **********************************/

/******************************  Переменные для работы с Serial портом  **********************************/
/// <summary>
/// Константа - Скорость Serial порта
/// </summary>
#define speedSerial 115200    
/******************************  /Переменные для работы с Serial портом **********************************/

/******************************  Переменные для работы передачи данных  **********************************/
/// <summary>
/// Константа - Символ новой строки
/// </summary>
#define newLine '\n'
/// <summary>
/// Константа - Символ начала пакета
/// </summary>
#define beginChar '?'
/// <summary>
/// Константа - Символ разделитель параметров пакета
/// </summary>
#define breakChar ';'
/// <summary>
/// Константа - Символ конца пакета
/// </summary>
#define endChar '!'
/// <summary>
/// Константа - Символ для очистки значения char
/// </summary>
#define clearChar '\0'

/// <summary>
/// Переменная хранящая последний полученный char
/// </summary>
char incomingChar;

/// <summary>
/// Переменная хранящая индекс текущего char в массиве хранящего пакет данных
/// </summary>
int indexCharInArray;

/// <summary>
/// Переменная хранящая завершено ли получения данных
/// </summary>
boolean finishReceiveData = false;

/// <summary>
/// Константа хранящая мак. размер массива хранящего пакет данных
/// </summary>
//#define receiveSizeArrayCOM 42
#define receiveSizeArrayCOM 142
/// <summary>
/// Массив хранящий пакет данных
/// </summary>
char receiveCharArrayCOM[receiveSizeArrayCOM];

/// <summary>
/// Переменная хранящее номер пакета в системе
/// </summary>
int globalNumberPacket = 0;

/// <summary>
/// Переменная хранящее номер символа начала пакета в пакете
/// </summary>
int indexBeginCharInPackage;
/// <summary>
/// Переменная хранящее номер символа конца пакета в пакете
/// </summary>
int indexEndCharInPackage;
/// <summary>
/// Переменная кол-во символов разделителя блоков данных в пакете
/// </summary>
int breakCharCountInPackage;

/// <summary>
/// Переменная хранящее номер пакета
/// </summary>
String numberPacket;
/// <summary>
/// Переменная хранящее длину пакета
/// </summary>
String lenPacket; 
/// <summary>
/// Переменная хранящее тип операции пакета
/// </summary>
String operationTypePacket;
/// <summary>
/// Переменная хранящее тип параметра пакета
/// </summary>
String parametrPacket;
/// <summary>
/// Переменная хранящее тип данных параметра пакета
/// </summary>
String dataTypePacket;
/// <summary>
/// Переменная хранящее значение параметра пакета
/// </summary>
String valuePacket;
/// <summary>
/// Переменная хранящее значение echo и нужен ли возврат
/// 9 - возврат, 1 отправка без подтверждения, 0 отправка с подтверждением
/// </summary>
String echoPacket;

/// <summary>
/// Переменная хранящее номер пакета
/// </summary>
int receiveNumberIntPacket = 0;
/// <summary>
/// Переменная хранящее длину пакета
/// </summary>
int receiveLenIntPacket;
/// <summary>
/// Переменная хранящее тип операции пакета
/// </summary>
int receiveOperationTypeIntPacket;

/// <summary>
/// Переменная хранящее реальную длину пакета, проверка после получения
/// </summary>
int receiveLength;

/// <summary>
/// Переменная хранящая последний отправленный пакет
/// </summary>
String lastSendData = "";

/// <summary>
/// Переменная хранящая нужно ли дожидаться подтверждения 
/// </summary>
boolean sendDataNeedEcho = false;
/******************************  /Переменные для работы передачи данных  **********************************/

/******************************  Переменные клавиатуры  **********************************/
// Состояние кнопок
char kbKey; // Нажатая клавиша, при проверке
byte kbKeyState; // Состояние последней используемой клавиши, при проверке


char customKey; // Получаем нажатую клавишу, для вызова KeypadEvent
/******************************  /Переменные клавиатуры  **********************************/

/******************************  Переменные запуска ТПА  **********************************/
/// <summary>
/// Переменная хранящее состояние включена ли проверка, к какой ардуине произошло подключение
/// </summary>
boolean checkWhatKindArduinConnected; 

/// <summary>
/// Переменная хранящее промежуток между отправкой на компьютер данных, говорящих о том, какая ардуина подключена
/// </summary>
int timeCheckWhatKindArduinConnected = 500;

/// <summary>
/// Переменная хранящее время предыдущей отправки на компьютер данных, говорящих о том, какая ардуина подключена
/// </summary>
long previousTimeCheckWhatKindArduinConnected = 0; 

/// <summary>
/// Переменная хранящее состояние ТПА, подключен ли к компьютеру и установлено ли соединение
/// </summary>
boolean connectWithTPAPC = false; 

/// <summary>
/// Переменная хранящее состояние переменной отправлена ли команда инициализации на PC
/// </summary>
boolean sendBeginInitializationTPA = false;

/// <summary>
/// Переменная хранящее состояние программы инициализирован ли ТПА
/// </summary>
boolean beginInitializationTPA = false;

/// <summary>
/// Переменная хранящее состояние переменной отправлена ли команда готов на PC
/// </summary>
boolean sendBeginWorkTPA = false;

/// <summary>
/// Переменная хранящее состояние ТПА, готов ли к работе или нет
/// </summary>
boolean beginWorkTPA = false; 

/// <summary>
/// Переменная хранящее промежуток между отправкой на компьютер данных, говорящих о том, что соединение есть
/// </summary>
int timeCheckConnected = 500;

/// <summary>
/// Переменная хранящее время предыдущей отправки на компьютер данных, говорящих о том, что соединение есть
/// </summary>
long previousTimeCheckConnected = 0; 

/******************************  /Переменные запуска ТПА  **********************************/

/******************************  Переменные отладки  **********************************/
unsigned long beginTimeCycle = 0;

/// <summary>
/// Переменная хранящее время старта цикла, для проверки скорости выполнения цикла
/// </summary>
unsigned long timeCycle = 0;

unsigned long maxTimeCycle = 0;

unsigned long resultGetCycleTime = 0;

unsigned long unsignedLongMax = 4294967295;

unsigned long digitalOne = 1;

// TODO: Добавить описание
//#define IS_DEBUG true
/******************************  /Переменные отладки  **********************************/

/// <summary>
/// Настройка программы
/// </summary>
void setup ()
{
	// Ускоряем работу ADC
	#if FASTADC
	// Устанавливаем делитель равный 16
	sbi(ADCSRA,ADPS2);
	cbi(ADCSRA,ADPS1);
	cbi(ADCSRA,ADPS0);
	#endif
	// /Ускоряем работу ADC

	// Выбор типа соединения для настройки
	switch (connectionType) 
	{
		case CT_Serial:
			// Serial порт в обе стороны
			Serial.begin(speedSerial);
            break;
	}	

	// Выходы Шим
	pinMode(s50in, OUTPUT);  // s50 Пропорциональный клапан, установка значения - расход V
	pinMode(s51in, OUTPUT);  // s51 Пропорциональный клапан, установка значения - давление Р
	pinMode(s26in, OUTPUT);  // s26 Пропорциональный клапан, установка значения - подпор при загрузке 
	// / Выходы Шим

	// Аналоговые входы
	pinMode(s50out, INPUT); // s50 Пропорциональный клапан, получение значения - расход V
	pinMode(s51out, INPUT); // s51 Пропорциональный клапан, получение значения - давление Р
	pinMode(s26out, INPUT); // s26 Пропорциональный клапан, получение значения - подпор при загрузке 

	pinMode(b1, INPUT); // Линейка узла запирания (b22, b23, b24r, b24l, b43, b44, b45, b46, b47)
	pinMode(b2, INPUT); // Линейка узла впрыска (b26, b27, b28, b29r, b29l)
	pinMode(b3, INPUT); // Линейка узла толкателей (b30, b31)
	// /Аналоговые входы

	// Цифровые выходы
	pinMode(s2, OUTPUT);	// s2 – запирание формы
	
	pinMode(s3, OUTPUT);	// s3 – раскрытие формы
	pinMode(s9, OUTPUT);    // s9 – силовое запирание   TODO: правда ли? 

	pinMode(s4, OUTPUT);	// s4 – подвод сопла 
	pinMode(s5, OUTPUT);	// s5 – отвод сопла 

	pinMode(s6, OUTPUT);	// s6 – впрыск 
	pinMode(s7, OUTPUT);	// s7 – отвод шнека
	pinMode(s8, OUTPUT);    // s8 – загрузка 

	pinMode(s56, OUTPUT);	// s56 – нагрузка 2-й камеры насоса TODO: правда ли?
	pinMode(s74, OUTPUT);	// s74 – давление защиты формы TODO: правда ли?

	pinMode(s33, OUTPUT);	// s33 – сброс давления запирания 
	
	pinMode(s52, OUTPUT);	// s52 – разогрев масла TODO: правда ли?

	pinMode(s10, OUTPUT);	// s10 – толкатель вперед 
	pinMode(s10b, OUTPUT);	// s10b – толкатель назад 

	//pinMode(s47, OUTPUT);	// s47 – впрыск с гидроаккумулятором 
	// /Цифровые выходы

	// Цифровые входы
	pinMode(b63, INPUT);	// b63 – проверка, разогрета ли масса

	// / Цифровые входы

	// /Порты

	// Прерывания двери
	attachInterrupt(4, CheckDoorInterrupt, CHANGE); // b20 – концевик двери, 1 линия 19 pin
	attachInterrupt(5, CheckDoorSecondInterrupt, CHANGE); // b20b – концевик двери, 2 линия 18 pin	
	// /Прерывания двери

	// Добавляем событие на нажатие клавиатуры
	customKeypad.addEventListener(keypadEvent);

	// Выставляем параметры выходов в начальное положение
	FullStopTPA();

	// Пауза 2 секунд, ждем инициализацию модулей
	//delay(2000);
	_delay_ms(2000);
}

/// <summary>
/// Основной цикл
/// </summary>
void loop()
{   
	// TODO: Добавить описание
        beginTimeCycle = micros();
        
	// Выбираем протокол передачи данных
	switch (connectionType) 
	{
		case CT_Serial:
			// Протокол передачи данных на основе Serial порта

			// Получение данных с Serial порта
			GetReceiveDataFromSerial();

			// Завершено ли получения данных
			if (finishReceiveData)
			{
				// Парсинг полученных данных
				ParseReceiveData();

				ClearReceiveDataVariable();
			}
			break;
	}

	// Проверка к какой ардуине произошло подключение
	if (checkWhatKindArduinConnected)
	{
		if (previousTimeCheckWhatKindArduinConnected == 0)
		{
			previousTimeCheckWhatKindArduinConnected = millis();
		}

		if (CheckTimerMillis(previousTimeCheckWhatKindArduinConnected, timeCheckWhatKindArduinConnected))
		{
			SendDataFromSerial("General Arduino", OT_CheckWhatKindArduinConnected, P_None, DT_Int, 1);
			previousTimeCheckWhatKindArduinConnected = millis();
		}
		
		return;
	}

	// Если ардуина соединена с PC, но переменные не инициализированы 
	if (connectWithTPAPC && !beginInitializationTPA)
	{
		// Отправка команды на начала инициализации
		if (!sendBeginInitializationTPA)
		{
			SendDataFromSerial("InitArduinoStart", OT_InitArduinoStart, P_None, DT_Int, 0);
			sendBeginInitializationTPA = true;	
		}
		
		return;
	}

	// Если ардуина соединена с PC, переменные инициализированы, но ТПА не готов к работе
	if (connectWithTPAPC && beginInitializationTPA && !beginWorkTPA)
	{
		if (!sendBeginWorkTPA)
		{
			SendDataFromSerial("TPAReadyToWork", OT_TPAReadyToWork, P_None, DT_Int, 0);	
			sendBeginWorkTPA = true;
		}

		return;
	}

	if (connectWithTPAPC && beginInitializationTPA && beginWorkTPA)
	{
		// Выбираем тип работы ТПА         
		switch (tPAWorkEnum) 
		{
			case TPAW_Automatic:
				// Автоматический режим работы
				TPAWAutomatic();		
				break;
			case TPAW_SemiAutomatic:
				// Полуавтоматический режим работы
				TPAWSemiAutomatic();
				break;
			case TPAW_Manual:
				// Ручной режим работы
				TPAWManual();
				break;
		}

		if (previousTimeCheckConnected == 0)
		{
			previousTimeCheckConnected =  millis();
		}
		if(CheckTimerMillis(previousTimeCheckConnected, timeCheckConnected))
		{
			SendDataFromSerial("CheckProtocolDataTransfer", OT_CheckProtocolDataTransfer, P_None, DT_Int, 1);
			previousTimeCheckConnected =  millis();
		}
	}

        timeCycle = GetCycleTime(beginTimeCycle, micros());
  
        if (timeCycle > maxTimeCycle)
        {
          maxTimeCycle = timeCycle;
          //Serial.println(maxTimeCycle);
        }
}

/******************************  Функции прерываний **********************************/
/// <summary>
// Функция, прерывания открытия двери линия 1
/// </summary>
void CheckDoorInterrupt()
{
	// TODO: Сделать функцию CheckDoorInterrupt
	// TODO: Добавить описание
}

/// <summary>
// Функция, прерывания открытия двери линия 2
/// </summary>
void CheckDoorSecondInterrupt()
{
	// TODO: Сделать функцию CheckDoorSecondInterrupt
	// TODO: Добавить описание
}
/******************************  /Функции прерываний **********************************/

/******************************  Функции работы с таймерами **********************************/
/// <summary>
/// Проверка таймера на основе millis
/// </summary>
//boolean CheckTimerMillis(long PreviousTimmer, int Time)
//{
//	// Если предыдущие значение, больше 0
//	if (PreviousTimmer > 0)
//	{
//		// Получаем время с момента запуска программы и отнимем значение PreviousTimmer (время с начала запуска таймера). После это проверяем, прошло ли установленное время Time  
//		if ((millis() - PreviousTimmer) >= Time)
//		{
//			// Возвращаем true
//			return true;
//		}
//		else 
//		{
//			// Возвращаем false
//			return false;
//		}
//	}
//	else 
//	{
//		// Возвращаем false
//		return true;
//	}
//}

boolean CheckTimerMillis(long PreviousTimmer, int Time)
{
	// Если предыдущие значение, больше 0
	if (PreviousTimmer > 0)
	{
                unsigned long value = 0;
                unsigned long valueMillis = millis();
		// Получаем время с момента запуска программы и отнимем значение PreviousTimmer (время с начала запуска таймера). После это проверяем, прошло ли установленное время Time  
		if (valueMillis < PreviousTimmer)
                {
                        value = unsignedLongMax - PreviousTimmer + valueMillis + digitalOne;
                }
                else
                {
                        value = millis() - PreviousTimmer;
                }

                if (value >= Time)
		{
			// Возвращаем true
			return true;
		}
		else 
		{
			// Возвращаем false
			return false;
		}
	}
	else 
	{
		// Возвращаем false
		return false;
	}
}

unsigned long GetCycleTime(unsigned long BeginTime, unsigned long EndTime)
{
  if (EndTime<BeginTime)
  {
    resultGetCycleTime = unsignedLongMax - BeginTime + EndTime + digitalOne;
  }
  else
  {
    resultGetCycleTime = EndTime - BeginTime;
  }
  
  return resultGetCycleTime;
} 
/******************************  /Функции работы с таймерами **********************************/

/******************************  Функции работы с клавиатурой  **********************************/
/// <summary>
// Функция получает последнюю нажатую кнопку
/// </summary>
void GetCustomKey()
{
	customKey = customKeypad.getKey();
}

/// <summary>
// Функция блокировки, события на нажатия кнопки
/// </summary>
void keypadEventStop(KeypadEvent key){}

/// <summary>
// Функция проверки что и как нажато
/// </summary>
void keypadEvent(KeypadEvent key){
	// Получаем последнюю нажатую кнопку
	kbKey = key;
	
	// Получаем состояние последней нажатой кнопки
	kbKeyState = customKeypad.getState();

	// Если состояние последней кнопки не нажата
	if (kbKeyState == RELEASED)
	{
		// Проверяем режим работы TPA
		switch (tPAWorkEnum) 
		{
			case TPAW_Manual:
				// Ручной режим работы

				// Остановка ручного режима работы
				StopTPAWManual();
				break;
		}
	}
}

/// <summary>
// Функция, проверки нажата ли сейчас нужная кнопка и закрыта
/// </summary>
boolean CheckKeyPRESSEDOrHOLDAndCheckDoor(char checkKey)
{
	// Получаем последнюю нажатую кнопку
	GetCustomKey();

	// Проверяем нажата ли нужная кнопка сравнивая kbKey и checkKey
	// Так же проверяем нажата ли сейчас кнопка и закрыта ли дверь TPA
	return kbKey == checkKey && KeyPRESSEDOrHOLD() && CheckDoor();
}

/// <summary>
// Функция, проверки нажата ли сейчас нужная кнопка 
/// </summary>
boolean CheckKeyPRESSEDOrHOLD(char checkKey)
{
	// Получаем последнюю нажатую кнопку
	GetCustomKey();

	// Проверяем нажата ли нужная кнопка сравнивая kbKey и checkKey
	// Так же проверяем нажата ли сейчас кнопка
	return kbKey == checkKey && KeyPRESSEDOrHOLD();
}

/// <summary>
// Функция, проверки нажата ли сейчас кнопка 
/// </summary>
boolean KeyPRESSEDOrHOLD()
{
	// Получаем последнюю нажатую кнопку
	GetCustomKey();

	// Проверяем нажата ли сейчас кнопка
	return kbKeyState == PRESSED || kbKeyState == HOLD;
}
/******************************  /Функции работы с клавиатурой  **********************************/

/******************************  Функции проверки двери **********************************/
/// <summary>
/// Проверка открыта ли дверь
/// </summary>
boolean CheckDoor()
{
	// TODO: Добавить описание
	// TODO: Сделать на прерываниях
	/*if (!DigitalReadOne(b20) && DigitalReadOne(b20b))
	{
		// Дверь закрыта первый датчик 0, второй 1
		return true;	
	}
	else if (!DigitalReadOne(b20) && DigitalReadOne(b20b)
	{
		// Дверь открыта первый датчик 1, второй 0
		return false;
	}
	else 
	{
		// Ошибка в работе, запускается таймер, по истечению которого машина останавливается
		
	}*/

	// Возвращаем true
	return true;
}
/******************************  /Функции проверки двери **********************************/

/******************************  Функции включения железа с проверкой важных параметров	**********************************/
/// <summary>
/// Функция проверки и установки цифрового выхода в нужное значение
/// </summary>
void DigitalWriteOne(int pin, boolean value)
{
	// Проверяем выход, если состояние выхода не равно value, устанавливаем выход в соответствии с значением value
	if (DigitalReadOne(pin) != value)
	{
		// Шлем отладочные данные
		#if(IS_DEBUG)
		String stringDebug = "DigitalWriteOne pin - ";
		stringDebug += pin;
		stringDebug += " - value - ";
		stringDebug += value;

		SendDebugData(stringDebug);
		#endif

		// Устанавливаем выход в соответствии с значением value
		digitalWrite(pin, value);
	}
}

/// <summary>
/// Функция проверки и установки аналогового выхода в нужное значение
/// </summary>
void AnalogWriteOne(int pin, int value)
{
	// Проверяем выход, если состояние выхода не равно value, устанавливаем выход в соответствии с значением value
	if (AnalogReadOne(pin) != value)
	{
		// Шлем отладочные данные
		#if(IS_DEBUG)
		String stringDebug = "AnalogWriteOne pin - ";
		stringDebug += pin;
		stringDebug += " - value - ";
		stringDebug += value;

		SendDebugData(stringDebug);
		#endif

		// Устанавливаем выход в соответствии с значением value
        analogWrite(pin, value);
	}
}

/// <summary>
/// Функция проверки и установки цифрового выхода в нужное значение, если дверь закрыта
/// </summary>
void DigitalWriteWithCheckDoor(int pin, boolean value)
{
	// Проверка открыта ли дверь
    if (CheckDoor())
    {
		// Установка цифрового выхода в нужное значение
        DigitalWriteOne(pin, value);
    }
}

/// <summary>
/// Функция проверки и установки аналогового выхода в нужное значение, если дверь закрыта
/// </summary>
void AnalogWriteWithCheckDoor(int pin, int value)
{
	// Проверка открыта ли дверь
    if (CheckDoor())
    {
		// Установка аналогового выхода в нужное значение
        AnalogWriteOne(pin, value);
    }
}

/// <summary>
/// Функция проверки и установки цифрового выхода в нужное значение, если дверь закрыта и нажата кнопка
/// </summary>
void DigitalWriteWithCheckDoorAndHoldKey(int pin, boolean value, char key)
{
	// Проверка нажата ли кнопка 
	if (CheckKeyPRESSEDOrHOLD(key))
	{
		// Проверка и установка цифрового выхода в нужное значение, если дверь закрыта
		DigitalWriteWithCheckDoor(pin, value);
	}
}

/// <summary>
/// Функция проверки и установки аналогового выхода в нужное значение, если дверь закрыта и нажата кнопка
/// </summary>
void AnalogWriteWithCheckDoorAndHoldKey(int pin, int value, char key)
{
	// Проверка нажата ли кнопка 
	if (CheckKeyPRESSEDOrHOLD(key))
	{
		// Проверка и установка аналогового выхода в нужное значение, если дверь закрыта
		AnalogWriteWithCheckDoor(pin, value);
	}
}

bool DigitalReadOne(int Pin)
{
	return digitalRead(Pin);
}

int AnalogReadOne(int Pin)
{
	return analogRead(Pin);
}

/******************************  /Функции включения железа с проверкой важных параметров	**********************************/

/******************************  Функции режимов ТПА  **********************************/
/// <summary>
/// Автоматический режим работы
/// </summary>
void TPAWAutomatic()
{

}

/// <summary>
/// Полуавтоматический режим работы
/// </summary>
void TPAWSemiAutomatic()
{

}

/// <summary>
/// Ручной режим работы
/// </summary>
void TPAWManual()
{
	// Проверка нажата ли какая-та кнопка и дверь закрыта
	if (!KeyPRESSEDOrHOLD() || !CheckDoor())
	{
		// Если дверь не закрыта или не нажата какая-то кнопка обрываем выполнение
		return;
	}
	
	// Нажата ли кнопка 1 и закрыта ли дверь
	if (CheckKeyPRESSEDOrHOLDAndCheckDoor('1'))
	{	
		// Нажата 1

		// Изменение типа шага в ручном цикле работы ТПА
		ChangeManualStepCycle(TPAMSC_CloseForm);

		// Смыкание формы
		CloseFormTPAWManual();
	}
	// Нажата ли кнопка 2 и закрыта ли дверь
	else if (CheckKeyPRESSEDOrHOLDAndCheckDoor('2'))
	{
		// Нажата 2

		// Изменение типа шага в ручном цикле работы ТПА
		ChangeManualStepCycle(TPAMSC_SupplyNozzle);

		// Подвод сопла
		SupplyNozzleTPAWManual();
	}
	// Нажата ли кнопка 3 и закрыта ли дверь
	else if (CheckKeyPRESSEDOrHOLDAndCheckDoor('3'))
	{
		// Нажата 3

		// Изменение типа шага в ручном цикле работы ТПА
		ChangeManualStepCycle(TPAMSC_SupplyAuger);

		// Подвод шнека
		SupplyAugerTPAWManual();
	}
	// Нажата ли кнопка 4 и закрыта ли дверь
	else if (CheckKeyPRESSEDOrHOLDAndCheckDoor('4'))
	{
		// Нажата 4

		// Изменение типа шага в ручном цикле работы ТПА
		ChangeManualStepCycle(TPAMSC_Injection);

		// Впрыск
		InjectionTPAWManual();
	}
	// Нажата ли кнопка 5 и закрыта ли дверь
	else if (CheckKeyPRESSEDOrHOLDAndCheckDoor('5'))
	{
		// Нажата 5

		// Изменение типа шага в ручном цикле работы ТПА
		ChangeManualStepCycle(TPAMSC_LoadFromBunker);

		// Загрузка с бункера
		LoadFromBunkerTPAWManual();
	}
	// Нажата ли кнопка 6 и закрыта ли дверь
	else if (CheckKeyPRESSEDOrHOLDAndCheckDoor('6'))
	{
		// Нажата 6

		// Изменение типа шага в ручном цикле работы ТПА
		ChangeManualStepCycle(TPAMSC_DiversionAuger);

		// Отвод шнека
		DiversionAugerTPAWManual();
	}
	// Нажата ли кнопка 7 и закрыта ли дверь
	else if (CheckKeyPRESSEDOrHOLDAndCheckDoor('7'))
	{
		// Нажата 7

		// Изменение типа шага в ручном цикле работы ТПА
		ChangeManualStepCycle(TPAMSC_DiversionNozzle);

		// Отвод сопла
		DiversionNozzleTPAWManual();
	}
	// Нажата ли кнопка 8 и закрыта ли дверь
	else if (CheckKeyPRESSEDOrHOLDAndCheckDoor('8'))
	{
		// Нажата 8

		// Изменение типа шага в ручном цикле работы ТПА
		ChangeManualStepCycle(TPAMSC_OpenForm);

		// Открытие формы
		OpenFormTPAWManual();
	}
	// Нажата ли кнопка 9 и закрыта ли дверь
	else if (CheckKeyPRESSEDOrHOLDAndCheckDoor('9'))
	{
		// Нажата 9

		// Изменение типа шага в ручном цикле работы ТПА
		ChangeManualStepCycle(TPAMSC_PlungerForward);

		// Толкатель вперёд
		PlungerForwardTPAWManual();
	}
	// Нажата ли кнопка 0 и закрыта ли дверь
	else if (CheckKeyPRESSEDOrHOLDAndCheckDoor('0'))
	{
		// Нажата 0

		// Изменение типа шага в ручном цикле работы ТПА
		ChangeManualStepCycle(TPAMSC_PusherAgo);

		// Толкатель назад
		PusherAgoTPAWManual();
	}
	// Нажата ли кнопка A и закрыта ли дверь
	else if (CheckKeyPRESSEDOrHOLDAndCheckDoor('A'))
	{
		// Нажата A
		// Линейка узла запирания (b22, b23, b24r, b24l, b43, b44, b45, b46, b47)

		// Шлем отладочные данные
		#if(IS_DEBUG)
		SendDebugData(String(AnalogReadOne(b1)));
		#endif
	}
	// Нажата ли кнопка B и закрыта ли дверь
	else if (CheckKeyPRESSEDOrHOLDAndCheckDoor('B'))
	{
		// Нажата B
		// Линейка узла впрыска (b26, b27, b28, b29r, b29l)

		// Шлем отладочные данные
		#if(IS_DEBUG)
		SendDebugData(String(AnalogReadOne(b2)));
		#endif
	}
	// Нажата ли кнопка C и закрыта ли дверь
	else if (CheckKeyPRESSEDOrHOLDAndCheckDoor('C'))
	{
		// Нажата C
		// Линейка узла толкателей (b30, b31)

		// Шлем отладочные данные
		#if(IS_DEBUG)
		SendDebugData(String(AnalogReadOne(b3)));
		#endif
	}
	// Нажата ли кнопка * и закрыта ли дверь
	else if (CheckKeyPRESSEDOrHOLDAndCheckDoor('*'))
	{
		// Нажата *
		// s50 Пропорциональный клапан, получение значения - расход V

		// Шлем отладочные данные
		#if(IS_DEBUG)
		SendDebugData(String(AnalogReadOne(s50out)));
		#endif
	}
	// Нажата ли кнопка # и закрыта ли дверь
	else if (CheckKeyPRESSEDOrHOLDAndCheckDoor('#'))
	{
		// Нажата #
		// s51 Пропорциональный клапан, получение значения - давление Р

		// Шлем отладочные данные
		#if(IS_DEBUG)
		SendDebugData(String(AnalogReadOne(s51out)));
		#endif
	}
	// Нажата ли кнопка D и закрыта ли дверь
	else if (CheckKeyPRESSEDOrHOLDAndCheckDoor('D'))
	{
		// Нажата D
		// s26 Пропорциональный клапан, получение значения - подпор при загрузке 

		// Шлем отладочные данные
		#if(IS_DEBUG)
		SendDebugData(String(AnalogReadOne(s26out)));
		#endif
	}
	else 
	{
		// Нажата кнопка, которой нет в списке обработки

		// Полная остановка ТПА
		FullStopTPA();

		// Изменение типа шага в ручном цикле работы ТПА
		ChangeManualStepCycle(TPAMSC_None);
	}
}
/******************************  Функции режимов ТПА  **********************************/

/******************************  Функции ручного режима ТПА  **********************************/
/// <summary>
/// Изменение типа шага в ручном цикле работы ТПА
/// </summary>
void ChangeManualStepCycle(TPAManualStepCycleEnum ManualStep)
{
	// Если текущий тип шага не равен новому значению, присваиваем новое значение
	if (tPAManualStepCycleEnum != ManualStep)
	{
		// Присваиваем новое значение
		tPAManualStepCycleEnum = ManualStep;
	}
}

/// <summary>
/// Смыкание формы
/// </summary>
void CloseFormTPAWManual()
{	
	// Шлем отладочные данные
	#if(IS_DEBUG)
	SendDebugData("CloseFormTPAWManual");
	#endif

	// Нажата ли кнопка 1 и закрыта ли дверь
	if (CheckKeyPRESSEDOrHOLDAndCheckDoor('1'))
	{
		// Шлем отладочные данные
		#if(IS_DEBUG)
		SendDebugData("CloseForm");
		#endif

		/*
		// Шлем отладочные данные
		#if(IS_DEBUG)
		SendDebugData(String(AnalogReadOne(b1)));
		#endif

		// Шлем отладочные данные
		#if(IS_DEBUG)
		SendDebugData(String(b44));
		#endif

		// Шлем отладочные данные
		#if(IS_DEBUG)
		SendDebugData(String(b22));
		#endif

		// Шлем отладочные данные
		#if(IS_DEBUG)
		SendDebugData(String(b46));
		#endif

		// Шлем отладочные данные
		#if(IS_DEBUG)
		SendDebugData(String(b23));
		#endif
		*/

		// Проверка, открыта ли форма и не началось предохранение пресс-формы
		// b44 концевик, ограничивает движение открытия пресс-формы, наверно и закрытие начинается с данного концевика
		// b22 концевик, начала предохранения пресс-формы
		if (AnalogReadOne(b1) >= b44 && AnalogReadOne(b1) < b22)
		{
			/*
			// Шлем отладочные данные
			#if(IS_DEBUG)
			SendDebugData("CloseForm_1");
			#endif
			*/	

			// Устанавливаем значение V1
			// Скорость замыкания пресс-формы
			AnalogWriteWithCheckDoorAndHoldKey(s50in, v1, '1');
			
			// Запускаем запирание формы s2 включено
			// S2 электро магнит запирания формы
			DigitalWriteWithCheckDoorAndHoldKey(s2, HIGH, '1');
		}

		// Началось предохранение пресс-формы и не началось затухание закрытие пресс-формы
		// b22 концевик, начала предохранения пресс-формы
		// b46 концевик, затухание закрытие пресс-формы
		if (AnalogReadOne(b1) >= b22 && AnalogReadOne(b1) < b46)
		{
			/*
			// Шлем отладочные данные
			#if(IS_DEBUG)
			SendDebugData("CloseForm_2");
			#endif
			*/	

			// Устанавливаем значение P1
			// Давление предохранения пресс-формы
			AnalogWriteWithCheckDoorAndHoldKey(s51in, p1, '1');

			// Запускается таймер t7
			// Таймер защиты формы
			// Если previousT7 равно 0
			if (previousT7 == 0)
			{
				/*
				// Шлем отладочные данные
				#if(IS_DEBUG)
				SendDebugData("CloseForm_3");
				#endif
				*/
				
				// Устанавливаем таймер t7
				previousT7 = millis();	
			}
		}

		// Началось затухания закрытия пресс-формы, но не закончилось закрытие формы
		// b46 концевик, затухание закрытие пресс-формы
		// b23 концевик, конец затухания закрытия пресс-формы, пресс-формы закрыта
		if (AnalogReadOne(b1) >= b46 && AnalogReadOne(b1) < b23)
		{
			/*
			// Шлем отладочные данные
			#if(IS_DEBUG)
			SendDebugData("CloseForm_4");
			#endif
			*/

			// Устанавливаем значение V2
			// 2-Скорость замыкания пресс-формы
			AnalogWriteWithCheckDoorAndHoldKey(s50in, v2, '1');
		}

		// Если форма закрыта или сработал таймер защиты пресс-формы
		// b23 концевик, конец затухания закрытия пресс-формы, пресс-формы закрыта
		// CheckTimerMillis(previousT7, t7) таймер защиты пресс-формы
		if (AnalogReadOne(b1) >= b23  ||
			CheckTimerMillis(previousT7, t7))
		{
			/*
			// Шлем отладочные данные
			#if(IS_DEBUG)
			SendDebugData("CloseForm_5");
			#endif
			*/

			// Если сработал таймер t7
			// Таймер защиты пресс-формы
			if (CheckTimerMillis(previousT7, t7))
			{
				// Ошибка ТПА
				Errore("Закрытие формы. Сработал таймер t7, раньше чем форма была закрыта.", 0);
			}
			else
			{
				// Включение удержания формы

				// Запускаем силовое запирание формы s9 включено
				DigitalWriteWithCheckDoorAndHoldKey(s9, HIGH, '1');

				// Разогрев масла ? s52 включено
				DigitalWriteWithCheckDoorAndHoldKey(s52, HIGH, '1');
			}
		} 
	}	
}

/// <summary>
/// Подвод сопла
/// </summary>
void SupplyNozzleTPAWManual()
{
	// TODO: Установка переменой withSupplyNozzle с компьютера  

	// Шлем отладочные данные
	#if(IS_DEBUG)
	SendDebugData("SupplyNozzleTPAWManual()");
	#endif

	/*
	// Шлем отладочные данные
	#if(IS_DEBUG)
	SendDebugData(String(AnalogReadOne(b1)));
	#endif

	// Шлем отладочные данные
	#if(IS_DEBUG)
	SendDebugData(String(AnalogReadOne(b2)));
	#endif

	// Шлем отладочные данные
	#if(IS_DEBUG)
	SendDebugData(String(b23));
	#endif

	// Шлем отладочные данные
	#if(IS_DEBUG)
	SendDebugData(String(b28));
	#endif*/

	// Нажата ли кнопка 2 и закрыта ли дверь
	if (CheckKeyPRESSEDOrHOLDAndCheckDoor('2'))
	{
		// Проверяем включен ли режим с подвода сопла
		if (WithOutSupplyNozzle())
		{
			// Проверяем, закрыта ли форма и концевик останова перемещения назад, он должен быть равен или меньше
			// b28 концевик, останов перемещения сопла назад 
			// b23 концевик, конец затухания закрытия пресс-формы, пресс-формы закрыта
			if (AnalogReadOne(b2) >= b28 && AnalogReadOne(b1) >= b23)
			{
				// Запускаем подвод сопла s4 включен
				DigitalWriteWithCheckDoorAndHoldKey(s4, HIGH, '2');		

				// Запускается таймер t8
				// Таймер замедления до подвода шнека
				// Если previousT8 равно 0
				if (previousT8 == 0)
				{
					// Устанавливаем таймер t8
					previousT8 = millis();	
				}

				// Если сработал таймер t8
				if (CheckTimerMillis(previousT8, t8))
				{
					// Остановка подвода сопла s4 отключен
					DigitalWriteWithCheckDoorAndHoldKey(s4, LOW, '2');	
				}
			}
		}
	}
}

/// <summary>
/// Функции проверки подвода сопла, можно ли его подводить, отводить
/// </summary>
boolean WithOutSupplyNozzle()
{
	// Если b28 и b27 не равны и установлен режим подвода сопла 
	// b27 концевик, завершения дозирования, старт перемещения шнека назад
	// b28 концевик, останов перемещения сопла назад 
	if (b27 != b28 && withSupplyNozzle)
	{
		// Возвращаем true
		return true;
	}
	else 
	{
		// Если b28 и b27 равны
		// b27 концевик, завершения дозирования, старт перемещения шнека назад
		// b28 концевик, останов перемещения сопла назад 
		if (b27 == b28)
		{
			// Ошибка ТПА
			Errore("Концевик, b27 и b28 равны, подвод сопла не возможен.", 0);
		}
		else 
		{
			// Ошибка ТПА
			Errore("Включен режим без подвода сопла.", 0);
		}

		// Возвращаем false
		return false;
	}
}

/// <summary>
/// Подвод шнека
/// </summary>
void SupplyAugerTPAWManual()
{
	// Шлем отладочные данные
	#if(IS_DEBUG)
	SendDebugData("SupplyAugerTPAWManual()");
	#endif

	// b63 – проверка, разогрета ли масса
	// b23 концевик, конец затухания закрытия пресс-формы, пресс-формы закрыта
	if (CheckKeyPRESSEDOrHOLDAndCheckDoor('3') && DigitalReadOne(b63) && AnalogReadOne(b1) >= b23)
	{
		// Запускаем подвод шнека 
		DigitalWriteWithCheckDoorAndHoldKey(s6, HIGH, '3');
	}
}

/// <summary>
/// Впрыск
/// </summary>
void InjectionTPAWManual()
{
	// TODO: Есть сомнения как работает, проверить

	// TODO: Когда стартует v30percent
	// TODO: Что делать, когда таймер t4 сработал

	// Шлем отладочные данные
	#if(IS_DEBUG)
	SendDebugData("InjectionTPAWManual()");
	#endif

	// Нажата ли кнопка 4 и закрыта ли дверь
	if (CheckKeyPRESSEDOrHOLDAndCheckDoor('4'))
	{
		// b63 – проверка, разогрета ли масса
		// b23 концевик, конец затухания закрытия пресс-формы, пресс-формы закрыта
		// b29r концевик, остановка возврата шнека, наверно и старт шнека начинается от b29r
		if (DigitalReadOne(b63) && AnalogReadOne(b1) >= b23 && AnalogReadOne(b2) >= b29r)
		{
			// Устанавливаем значение P2
			// 1-я ступень давления
			AnalogWriteWithCheckDoorAndHoldKey(s51in, p2, '4');

			// Устанавливаем значение V3
			// 1-я скорость впрыска
			AnalogWriteWithCheckDoorAndHoldKey(s50in, v3, '4');

			// Запускаем подвод шнека s6 
			DigitalWriteWithCheckDoorAndHoldKey(s6, HIGH, '4');

			// Запускается таймер t2
			if (previousT2 == 0)
			{
				// Устанавливаем таймер t2
				previousT2 = millis();	
			}
		}

		// b63 – проверка, разогрета ли масса
		// b23 концевик, конец затухания закрытия пресс-формы, пресс-формы закрыта
		// b29l концевик, включение скорости, вызывает переключение 1 скорости впрыска на 2
		// b29l должен стоять слева от b27 иначе переключение с V3 на V4 не происходит
		if (DigitalReadOne(b63) && AnalogReadOne(b1) >= b23 && AnalogReadOne(b2) >= b29l
			&& b29l > b27)
		{
			// Устанавливаем значение V4
			// 2-я скорость впрыска
			AnalogWriteWithCheckDoorAndHoldKey(s50in, v4, '4');
		}

		// Если сработал таймер t2
		// b26 концевик, переключение впрыска с 1 ступени на 2 ступень
		// t3 начинает действовать после достижения b26
		if (CheckTimerMillis(previousT2, t2) && AnalogReadOne(b2) >= b26)
		{
			// Запускается таймер t3
			if (previousT3 == 0)
			{
				// Устанавливаем значение P3
				// 2-я ступень давления
				AnalogWriteWithCheckDoorAndHoldKey(s51in, p3, '4');

				// Устанавливаем таймер t3
				previousT3 = millis();	
			}
		}

		// Если таймер t3 включен и он сработал
		if (previousT3 > 0 && CheckTimerMillis(previousT3, t3))
		{
			// Запускается таймер t4
			if (previousT4 == 0)
			{
				// Устанавливаем значение P4
				// 3-я ступень давления
				AnalogWriteWithCheckDoorAndHoldKey(s51in, p4, '4');

				// Устанавливаем таймер t4
				previousT4 = millis();	
			}
		}

		// Если таймер t4 включен и он сработал
		if (previousT4 > 0 && CheckTimerMillis(previousT4, t4))
		{

		}
	}
}

/// <summary>
/// Загрузка с бункера
/// </summary>
void LoadFromBunkerTPAWManual()
{
	// Шлем отладочные данные
	#if(IS_DEBUG)
	SendDebugData("LoadFromBunkerTPAWManual()");
	#endif

	// Нажата ли кнопка 5 и закрыта ли дверь
	if (CheckKeyPRESSEDOrHOLDAndCheckDoor('5'))
	{
		// b63 – проверка, разогрета ли масса
		// b23 концевик, конец затухания закрытия пресс-формы, пресс-формы закрыта
		if (DigitalReadOne(b63) && AnalogReadOne(b1) >= b23)
		{
			
		}
	}
	
}

/// <summary>
/// Отвод шнека
/// </summary>
void DiversionAugerTPAWManual()
{
	// TODO: Добавить описание 
	// TODO: Доделать

	// Шлем отладочные данные
	#if(IS_DEBUG)
	SendDebugData("DiversionAugerTPAWManual()");
	#endif

	// b63 – проверка, разогрета ли масса
	// b23 – должно быть равно
	if (CheckKeyPRESSEDOrHOLDAndCheckDoor('6') && DigitalReadOne(b63) && AnalogReadOne(b1) >= b23)
	{
		// Запускаем отвод шнека 
		DigitalWriteWithCheckDoorAndHoldKey(s7, HIGH, '6');
	}
}


/// <summary>
/// Отвод сопла
/// </summary>
void DiversionNozzleTPAWManual()
{
	// TODO: Добавить описание 
	// TODO: Доделать

	// Шлем отладочные данные
	#if(IS_DEBUG)
	SendDebugData("DiversionNozzleTPAWManual()");
	#endif

	if (CheckKeyPRESSEDOrHOLDAndCheckDoor('7'))
	{
		// TODO: Надо ли проверять режим без подвода сопла
		if (WithOutSupplyNozzle())
		{
			// TODO: Точно ли не надо положение линейки по сравнению с концевиком b28
			if (AnalogReadOne(b2) >= b28)
			{
				// Запускаем oтвод сопла 
				DigitalWriteWithCheckDoorAndHoldKey(s5, HIGH, '7');
			}
		}
	}
}

/// <summary>
/// Открытие формы
/// </summary>
void OpenFormTPAWManual()
{
	// TODO: Добавить описание 
	// TODO: Доделать

	// Шлем отладочные данные
	#if(IS_DEBUG)
	SendDebugData("OpenFormTPAWManual()");
	#endif

	if (CheckKeyPRESSEDOrHOLDAndCheckDoor('8'))
	{
		// TODO: Доделать
	}
}

/// <summary>
/// Толкатель вперёд
/// </summary>
void PlungerForwardTPAWManual()	
{
	// TODO: Добавить описание 
	// TODO: Доделать

	// Шлем отладочные данные
	#if(IS_DEBUG)
	SendDebugData("PlungerForwardTPAWManual()");
	#endif

	if (CheckKeyPRESSEDOrHOLDAndCheckDoor('9'))
	{
		
	}
}

/// <summary>
/// Толкатель назад
/// </summary>
void PusherAgoTPAWManual()
{
	// TODO: Добавить описание 
	// TODO: Доделать

	// Шлем отладочные данные
	#if(IS_DEBUG)
	SendDebugData("PusherAgoTPAWManual()");
	#endif

	if (CheckKeyPRESSEDOrHOLDAndCheckDoor('0'))
	{
			
	}
}
/******************************  /Функции ручного режима ТПА  **********************************/

/******************************  Функции остановки ТПА  **********************************/
/// <summary>
/// Остановка ручного режима работы
/// </summary>
void StopTPAWManual()
{
	// TODO: Добавить описание

	// Шлем отладочные данные
	#if(IS_DEBUG)
	SendDebugData("StopTPAWManual()");
	#endif

	tPAManualStepCycleEnum = TPAMSC_None;

	// Полная остановка ТПА
	FullStopTPA();
}

/// <summary>
/// Полная остановка ТПА
/// </summary>
void FullStopTPA()
{
	// TODO: Добавить описание

	// Шлем отладочные данные
	#if(IS_DEBUG)
	SendDebugData("FullStopTPA()");
	#endif

	AnalogWriteOne(s50in, 255); // s50 Пропорциональный клапан, установка значения - расход V
	AnalogWriteOne(s51in, 255); // s51 Пропорциональный клапан, установка значения - давление Р
	AnalogWriteOne(s26in, 255); // s26 Пропорциональный клапан, установка значения - подпор при загрузке 

	DigitalWriteOne(s2, LOW);	// s2 – запирание формы
	DigitalWriteOne(s3, LOW);	// s3 – раскрытие формы
	DigitalWriteOne(s9, LOW);    // s9 – силовое запирание  TODO: правда ли? 

	DigitalWriteOne(s4, LOW);	// s4 – подвод сопла 
	DigitalWriteOne(s5, LOW);	// s5 – отвод сопла 

	DigitalWriteOne(s6, LOW);	// s6 – впрыск 
	DigitalWriteOne(s7, LOW);	// s7 – отвод шнека
	DigitalWriteOne(s8, LOW);    // s8 – загрузка 

	DigitalWriteOne(s56, LOW);	// s56 – нагрузка 2-й камеры насоса  TODO: правда ли?
	DigitalWriteOne(s74, LOW);	// s74 – давление защиты формы  TODO: правда ли?

	DigitalWriteOne(s33, LOW);	// s33 – сброс давления запирания 
	
	DigitalWriteOne(s52, LOW);	// s52 – разогрев масла TODO: правда ли?

	DigitalWriteOne(s10, LOW);	// s10 – толкатель вперед 
	DigitalWriteOne(s10b, LOW);	// s10b – толкатель назад 

	previousT1 = 0; // t1
	previousT2 = 0; // t2
	previousT3 = 0; // t3
	previousT4 = 0; // t4
	previousT5 = 0; // t5
	previousT6 = 0; // t6
	previousT7 = 0; // t7
	previousT8 = 0; // t8
	previousTl = 0; // tl

	//DigitalWriteOne(s47, LOW);	// s47 – впрыск с гидроаккумулятором 
}
/******************************  /Функции остановки ТПА  **********************************/

/******************************  Функции работы с Serial портом  **********************************/
/// <summary>
/// Получение данных с Serial порта
/// </summary>
void GetReceiveDataFromSerial()
{
	// TODO: Добавить описание 
	while(Serial.available())
	{
		/// <summary>
		/// Переменная хранящая последний полученный char
		/// </summary>
		char incomingChar = (char)Serial.read();

		if(incomingChar != newLine &&
			incomingChar != clearChar)
		{
			receiveCharArrayCOM[indexCharInArray] = incomingChar;
			indexCharInArray++;
		}
		else
		{
			receiveCharArrayCOM[indexCharInArray] = '\0';

			Serial.flush();

			finishReceiveData = true;
		}
	}
}

/// <summary>
/// Отправка данных по Serial порту
/// </summary>
void SendDataFromSerial(String SendValue, int OperationType, int Parametr, int DataType, int Echo)
{
	// TODO: Добавить описание
	globalNumberPacket++;

	RepeatSendDataFromSerial(SendValue, OperationType, Parametr, DataType, Echo);          
}

/// <summary>
/// TODO: Добавить описание 
/// </summary>
void SendConfirmationDataFromSerial(String SendValue, int OperationType, int Parametr, int DataType)
{
	// TODO: Добавить описание 
	RepeatSendDataFromSerial(SendValue, OperationType, Parametr, DataType, 9); 
}

/// <summary>
/// Повтор отправки данных по Serial порту
/// </summary>
void RepeatSendDataFromSerial(String SendValue, int OperationType, int Parametr, int DataType, int Echo)
{
	// TODO: Добавить описание 

	int sendValueLen = String(globalNumberPacket).length() +  String(OperationType).length() + String(Parametr).length() + String(DataType).length() + SendValue.length() + 9;
	sendValueLen += String(sendValueLen).length();
                  
	String sendPacket = beginChar + String(globalNumberPacket) + breakChar + String(sendValueLen) + breakChar + OperationType + breakChar + Parametr + breakChar + DataType + breakChar + SendValue + breakChar + Echo + endChar;     
    
	if (Echo == 0)
	{
		sendDataNeedEcho = true;
		lastSendData = sendPacket;
	}
                                
	SerialPrintLine(sendPacket);
}

void SerialPrintLine(String SendValue)
{
	SendValue += newLine;
	Serial.print(SendValue);
}
/******************************  /Функции работы с Serial портом  **********************************/

/******************************  Функции работы с полученными данными  **********************************/
/// <summary>
/// Очистка переменных получения данных, после получения данный
/// </summary>
void ClearReceiveDataVariable()
{
	// TODO: Добавить описание 
	indexCharInArray = 0;
	memset(receiveCharArrayCOM,clearChar,receiveSizeArrayCOM); 

	indexBeginCharInPackage = indexEndCharInPackage = breakCharCountInPackage = receiveLenIntPacket = receiveLength = receiveOperationTypeIntPacket = 0;
	numberPacket = lenPacket = operationTypePacket = parametrPacket = dataTypePacket = valuePacket = echoPacket = "";
		
	finishReceiveData = false;
}

/// <summary>
/// Парсинг полученных данных
/// </summary>
void ParseReceiveData()
{
	// Завершено ли получения данных
	if (finishReceiveData)
	{
		// Получаем длину пакета
		receiveLength = strlen(receiveCharArrayCOM);
		
		// Пробегаемся по массиву хранящий пакет данных
		for (int i = 0; i < receiveLength; i++)
		{
			switch (receiveCharArrayCOM[i]) {
				case beginChar:
					// Получаем индекс символ начала пакета в пакете
					indexBeginCharInPackage = i;
					break;
				case endChar:
					// Получаем индекс символ конца пакета в пакете
					indexEndCharInPackage = i;
					break;
				case breakChar:
					// Получаем кол-во символов разделителя блоков данных в пакете
					breakCharCountInPackage++;
					break;
				default:
					// В зависимости от номера блока данных в пакете, заполняем переменные блоков
					switch (breakCharCountInPackage) {
						case 0:
							// Номер пакета в системе
							numberPacket += receiveCharArrayCOM[i];
							break;
						case 1:
							// Длина пакета
							lenPacket += receiveCharArrayCOM[i];
							break;
						case 2:
							// Тип операции пакета
							operationTypePacket += receiveCharArrayCOM[i];
							break;
						case 3:
							// Тип параметра пакета
							parametrPacket += receiveCharArrayCOM[i];
							break;
						case 4:
							// Тип данных параметра пакета
							dataTypePacket += receiveCharArrayCOM[i];
							break;
						case 5:
							// Значение параметра пакета
							valuePacket += receiveCharArrayCOM[i];					
							break;
						case 6:
							// Значение параметра пакета
							echoPacket += receiveCharArrayCOM[i];					
							break;
					} 
					break;
			}
		}
       
		if (breakCharCountInPackage != 6)
		{
			ErroreDataReceived("1001",1001);
			Serial.println(receiveCharArrayCOM);
			return;
		}
		
		// Номер пакета в системе, Int
		receiveNumberIntPacket = numberPacket.toInt();
                
		// Длина пакета, Int
		receiveLenIntPacket = lenPacket.toInt();

		// Тип операции пакета, Int
		receiveOperationTypeIntPacket = operationTypePacket.toInt();
                               
		// Проверка корректности полученного пакета 
		if (receiveLength != receiveLenIntPacket // Реальная длина пакета должна быть равна значению блока длины пакета
                            || indexBeginCharInPackage != 0	// Индекс символа начала пакета, должен быть равен 0	
                            || indexEndCharInPackage != (receiveLength - 1) // Индекс символа конца пакета, должен быть равен значению блока длины пакета - 1
                            || (echoPacket == "9" && receiveNumberIntPacket != globalNumberPacket)
                            || (echoPacket != "9" && receiveNumberIntPacket != (globalNumberPacket + 1))
							)
        {
            ErroreDataReceived("1002",1002);

			return;
        }

		if (echoPacket == "0")
		{
			globalNumberPacket = receiveNumberIntPacket;
			SendConfirmationDataFromSerial(valuePacket, receiveOperationTypeIntPacket, parametrPacket.toInt(), dataTypePacket.toInt());
		}
		else if (echoPacket == "9")
		{
			if (sendDataNeedEcho)
            {
				String receiveStringCOM = String(receiveCharArrayCOM);
				receiveStringCOM.replace(";9!", ";0!");
				if (lastSendData == receiveStringCOM)
				{
					sendDataNeedEcho = false;
					lastSendData = "";
				}
				else
				{
					ErroreDataReceived("1003",1003);
				}
            }
		}
		else if (echoPacket != "9")
		{
			globalNumberPacket++;
		} 

		// Определяем тип операции пакета
		switch(receiveOperationTypeIntPacket)
		{
			case OT_CheckWhatKindArduinConnectedStart:
				checkWhatKindArduinConnected = true;

				break;
			case OT_CheckWhatKindArduinConnectedStop:
                                        
				checkWhatKindArduinConnected = false;

				globalNumberPacket = 0;

				break;
			case OT_ProtocolDataTransferStart:
				// Старт ТПА
					
				// Устанавливаем состояние программы запущена
				connectWithTPAPC = true;

				break;	
			case OT_InitArduinoWork:
				{
					int parametrIntPacket = parametrPacket.toInt();
					int dataTypeIntPacket = dataTypePacket.toInt();

					InitArduino(parametrIntPacket, dataTypeIntPacket, valuePacket);
				}
				break;
			case OT_InitArduinoStop:
				// Устанавливаем состояние переменной отправлена ли команда инициализации на PC
				sendBeginInitializationTPA = true;

				// Устанавливаем состояние программы инициализирован ли ТПА
				beginInitializationTPA = true;

				#if(IS_DEBUG)
				CheckInit();
				#endif

				break;
			case OT_TPAReadyToWork:
				// Устанавливаем состояние ТПА, готов ли к работе или нет
				beginWorkTPA = true; 

				break;
			case OT_ProtocolDataTransferStop:
				// Стоп ТПА
					
				globalNumberPacket = 0;

				// Устанавливаем состояние программы остановлена
				connectWithTPAPC = false;

				// Устанавливаем состояние переменной отправлена ли команда инициализации на PC
				sendBeginInitializationTPA = false;

				// Устанавливаем состояние программы инициализирован ли ТПА
				beginInitializationTPA = false;

				// Устанавливаем состояние переменной отправлена ли команда готов на PC
				sendBeginWorkTPA = false;
				
				// Устанавливаем состояние ТПА, готов ли к работе или нет
				beginWorkTPA = false; 

				break;
			default:				
				break;
		}         								
	}
	else 
	{
		ErroreDataReceived("1004",1004);
	}
}
/******************************  /Функции работы с полученными данными  **********************************/

/******************************  Функции инициализации Arduino   **********************************/
void InitArduino(int ParametrPacket, int DataTypePacket, String Value)
{
	switch(ParametrPacket)
	{
		case P_V1:
			v1 = Value.toInt();
			break;
		case P_V2:
			v2 = Value.toInt();
			break;
		case P_V3:
			v30percent = Value.toInt();
			break;
		case P_V4:
			v4 = Value.toInt();
			break;
		case P_V5:
			v5 = Value.toInt();
			break;
		case P_V6:
			v6 = Value.toInt();
			break;
		case P_V7:
			v7 = Value.toInt();
			break;
		case P_V8:
			v8 = Value.toInt();
			break;
		case P_V9:
			v9 = Value.toInt();
			break;
		case P_V10:
			v10 = Value.toInt();
			break;
		case P_V11:
			v11 = Value.toInt();
			break;
		case P_P1:
			p1 = Value.toInt();
			break;
		case P_P2:
			p2 = Value.toInt();
			break;
		case P_P3:
			p3 = Value.toInt();
			break;
		case P_P4:
			p4 = Value.toInt();
			break;
		case P_P5:
			p5 = Value.toInt();
			break;
		case P_P6:
			p6 = Value.toInt();
			break;
		case P_P7:
			p7 = Value.toInt();
			break;
		case P_P8:
			p8 = Value.toInt();
			break;
		case P_P9:
			p9 = Value.toInt();
			break;
		case P_P10:
			p10 = Value.toInt();
			break;
		case P_T1:
			t1 = Value.toInt();
			break;
		case P_T2:
			t2 = Value.toInt();
			break;
		case P_T3:
			t3 = Value.toInt();
			break;
		case P_T4:
			t4 = Value.toInt();
			break;
		case P_T5:
			t5 = Value.toInt();
			break;
		case P_T6:
			t6 = Value.toInt();
			break;
		case P_T7:
			t7 = Value.toInt();
			break;
		case P_T8:
			t8 = Value.toInt();
			break;
		case P_TL:
			tl = Value.toInt();
			break;
	}
}

void CheckInit()
{
	SendDebugData(String(v1));
	SendDebugData(String(v2));
	SendDebugData(String(v3));
	SendDebugData(String(v4));
	SendDebugData(String(v5));
	SendDebugData(String(v6));
	SendDebugData(String(v7));
	SendDebugData(String(v8));
	SendDebugData(String(v9));
	SendDebugData(String(v10));
	SendDebugData(String(v11));

	SendDebugData(String(p1));
	SendDebugData(String(p2));
	SendDebugData(String(p3));
	SendDebugData(String(p4));
	SendDebugData(String(p5));
	SendDebugData(String(p6));
	SendDebugData(String(p7));
	SendDebugData(String(p8));
	SendDebugData(String(p9));
	SendDebugData(String(p10));

	SendDebugData(String(t1));
	SendDebugData(String(t2));
	SendDebugData(String(t3));
	SendDebugData(String(t4));
	SendDebugData(String(t5));
	SendDebugData(String(t6));
	SendDebugData(String(t7));
	SendDebugData(String(t8));
	SendDebugData(String(tl));
}
/******************************  /Функции инициализации Arduino   **********************************/

/******************************  Функции работы с дебагом  **********************************/

/******************************  /Функции работы с дебагом  **********************************/

/******************************  Функции работы с ошибками  **********************************/
/// <summary>
/// Ошибка получения данных
/// </summary>
void ErroreDataReceived(String DataReceived, int TypeErrore)
{
	// TODO: Добавить описание 
	Errore(DataReceived, 0);
}

/// <summary>
/// Ошибка получения данных
/// </summary>
void Errore(String Data, int TypeErrore)
{
	// Полная остановка ТПА
	FullStopTPA();

	// Логирование ошибоки
	Log(Data, TypeErrore, true);
}
/******************************  /Функции работы с ошибками  **********************************/
void SendDebugData(String Data) 
{
	SendDataFromSerial(Data, OT_DebugMessageFromArduino, P_None, DT_Int, 1);
}
/******************************  Функции работы с логом  **********************************/
/// <summary>
/// Логирование действий или ошибок
/// </summary>
void Log(String Data, int TypeLog, bool isErrore)
{
	if (isErrore)
	{
		SendDataFromSerial(Data, OT_ErrorMessageFromArduino, P_None, DT_Int, TypeLog);
	}
}
/******************************  /Функции работы с логом  **********************************/


